package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.params.analyze.GradeBatchParam;
import com.qmth.distributed.print.business.bean.result.analyze.GradeBatchResult;
import com.qmth.distributed.print.business.entity.GradeBatch;
import com.qmth.distributed.print.business.enums.GradeAnalyzePaperStatusEnum;
import com.qmth.distributed.print.business.mapper.GradeBatchMapper;
import com.qmth.distributed.print.business.service.GradeBatchService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

/**
 * <p>
 * 分析-批次表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2022-05-20
 */
@Service
public class GradeBatchServiceImpl extends ServiceImpl<GradeBatchMapper, GradeBatch> implements GradeBatchService {

    @Override
    public IPage<GradeBatchResult> gradeBatchPage(String batchName, int pageNumber, int pageSize, SysUser requestUser) {
        batchName = SystemConstant.translateSpecificSign(batchName);
        return this.baseMapper.findGradeBatchPage(new Page<>(pageNumber, pageSize), batchName, requestUser.getSchoolId());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long editGradeBatch(GradeBatchParam gradeBatchParam, SysUser requestUser) {
        Long schoolId = requestUser.getSchoolId();
        Long id = gradeBatchParam.getId();
        String batchName = gradeBatchParam.getBatchName();
        Long batchTime = gradeBatchParam.getBatchTime();

        // 教研分析批次名称学校唯一
        GradeBatch check = this.getOne(new QueryWrapper<GradeBatch>().lambda().eq(GradeBatch::getSchoolId, schoolId).eq(GradeBatch::getBatchName, batchName));
        if (Objects.nonNull(check) && !Objects.equals(check.getId(), id)) {
            throw ExceptionResultEnum.ERROR.exception("已存在名称为： " + batchName + "，的教研分析批次");
        }

        GradeBatch gradeBatch = new GradeBatch();
        gradeBatch.setBatchName(batchName);
        gradeBatch.setBatchTime(batchTime);
        gradeBatch.setBatchCode(DateFormatUtils.format(batchTime, "yyyyMMdd"));
        gradeBatch.setStatus(GradeAnalyzePaperStatusEnum.SETTING_GRADE_PAPER);
        if (!SystemConstant.longNotNull(id)) {
            id = SystemConstant.getDbUuid();
        }
        gradeBatch.setId(id);

        this.saveOrUpdate(gradeBatch);
        return id;
    }
}
