package com.qmth.teachcloud.report.api;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.common.bean.params.BasicSemesterParams;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicSemesterService;
import com.qmth.teachcloud.common.util.AuthThirdUtil;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.report.business.bean.params.CourseParam;
import com.qmth.teachcloud.report.business.bean.params.TBExamParam;
import com.qmth.teachcloud.report.business.service.TBExamCourseService;
import com.qmth.teachcloud.report.business.service.TBExamService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.Optional;

/**
 * <p>
 * 教研分析开放接口前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2022-04-26
 */
@Api(tags = "教研分析开放接口Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.open}")
@Validated
public class OpenApiController {
    private final static Logger log = LoggerFactory.getLogger(OpenApiController.class);

    @Resource
    BasicSemesterService basicSemesterService;

    @Resource
    TBExamService tbExamService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    TBExamCourseService tbExamCourseService;

    @ApiOperation(value = "学期创建/更新接口")
    @ApiResponses({@ApiResponse(code = 200, message = "学期创建/更新接口", response = Object.class)})
    @RequestMapping(value = "/semester_edit", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result semesterEdit(@ApiParam(value = "接收学期数据信息", required = true) @RequestBody String result) throws IOException, InterruptedException, IllegalAccessException {
        Optional.ofNullable(result).orElseThrow(() -> ExceptionResultEnum.PARAMS_ERROR.exception("数据为空"));
        String decodeJson = URLDecoder.decode(result, SystemConstant.CHARSET_NAME);
        log.info("semesterEdit进来了,result:{}", decodeJson);
        BasicSemesterParams basicSemesterParams = JacksonUtil.readJson(decodeJson, BasicSemesterParams.class);
        Optional.ofNullable(basicSemesterParams).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("转换后的数据为空"));
        basicSemesterParams.validParams();
        BasicSchool basicSchool = AuthThirdUtil.hasPermission();
        return ResultUtil.ok(basicSemesterService.saveBasicSemesterCommon(basicSemesterParams, basicSchool.getId(), null));
    }

    @ApiOperation(value = "考试创建/更新接口")
    @ApiResponses({@ApiResponse(code = 200, message = "考试创建/更新接口", response = Object.class)})
    @RequestMapping(value = "/exam_edit", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result examEdit(@ApiParam(value = "接收考试数据信息", required = true) @RequestBody String result) throws IOException, InterruptedException {
        Optional.ofNullable(result).orElseThrow(() -> ExceptionResultEnum.PARAMS_ERROR.exception("数据为空"));
        String decodeJson = URLDecoder.decode(result, SystemConstant.CHARSET_NAME);
        log.info("examEdit进来了,result:{}", decodeJson);
        TBExamParam tbExamParam = JacksonUtil.readJson(decodeJson, TBExamParam.class);
        Optional.ofNullable(tbExamParam).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("转换后的数据为空"));
        tbExamParam.validParams();
        BasicSchool basicSchool = AuthThirdUtil.hasPermission();
        tbExamParam.setSchoolId(basicSchool.getId());
        return ResultUtil.ok(tbExamService.editTBExam(tbExamParam));
    }

    @ApiOperation(value = "分析课程（试卷）创建/更新接口")
    @ApiResponses({@ApiResponse(code = 200, message = "分析课程（试卷）创建/更新接口", response = Object.class)})
    @RequestMapping(value = "/course_edit", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    @Transactional
    public Result courseEdit(@ApiParam(value = "接收分析课程（试卷）数据信息", required = true) @RequestBody String result) throws IOException, InterruptedException {
        Optional.ofNullable(result).orElseThrow(() -> ExceptionResultEnum.PARAMS_ERROR.exception("数据为空"));
        String decodeJson = URLDecoder.decode(result, SystemConstant.CHARSET_NAME);
        log.info("courseEdit进来了进来了,result:{}", decodeJson);
        CourseParam courseParam = JacksonUtil.readJson(decodeJson, CourseParam.class);
        courseParam.validParams();
        BasicSchool basicSchool = AuthThirdUtil.hasPermission();
        basicCourseService.createCourse(basicSchool.getId(), courseParam.getCourseCode(), courseParam.getCourseName());
        tbExamCourseService.createCourse(basicSchool.getId(), courseParam.getExamId(), courseParam.getCourseCode(), courseParam.getCourseName(), courseParam.getPaperType(), courseParam.getTeachCollegeName());
        return ResultUtil.ok(true);
    }

//    @ApiOperation(value = "接收基础考务数据")
//    @ApiResponses({@ApiResponse(code = 200, message = "推送基础考务数据", response = Object.class)})
//    @RequestMapping(value = "/basic/examination/data", method = RequestMethod.POST)
//    public Result basicExaminationData(@ApiParam(value = "接收基础考务数据信息", required = true) @RequestBody String result) throws IOException, InterruptedException {
//        log.info("basicExaminationData进来了,result:{}", result);
//        String decodeJson = URLDecoder.decode(result, SystemConstant.CHARSET_NAME);
//        log.info("decodeJson:{}", decodeJson);
//        if (Objects.nonNull(decodeJson)) {
//            JSONObject jsonObject = JSONObject.parseObject(decodeJson);
//            String callbackUrl = jsonObject.getString("callbackUrl");
//            String accessKey = jsonObject.getString("accessKey");
//            String accessSecret = jsonObject.getString("accessSecret");
//            if (Objects.nonNull(callbackUrl)) {
//                Double progress = 0D;
//                for (int i = 0; i < 11; i++) {
//                    progress += 10D;
//                    JSONObject dataObjectJson = new JSONObject();
//                    dataObjectJson.put("progress", progress);
//                    if (progress == 100D) {
//                        dataObjectJson.put("status", "FINISHED");
//                    } else {
//                        dataObjectJson.put("status", "RUNNING");
//                    }
//                    Long timestamp = System.currentTimeMillis();
//                    String accessToken = SignatureEntity.build(SignatureType.SECRET, SystemConstant.METHOD, callbackUrl, timestamp, accessKey, accessSecret);
//                    String callbackResult = HttpUtil.postJson("http://127.0.0.1:7001" + callbackUrl, dataObjectJson.toJSONString(), accessToken, timestamp);
//                    if (Objects.nonNull(callbackResult)) {
//                        log.info("callbackResult:{}", JacksonUtil.parseJson(callbackResult));
//                    }
//                    Thread.sleep(5000);
//                }
//            }
//        }
//        return ResultUtil.ok(true);
//    }
}
